/***********************************************************************/
/*                                                                     */
/*   SVMTorch                                                          */
/*                                                                     */
/*   Author: Ronan Collobert                                           */
/*   Date: 21.07.2000                                                  */
/*                                                                     */
/*   Copyright (c) 2000  IDIAP - All rights reserved                   */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/
  

#ifndef INC_CHUNKER
#define INC_CHUNKER

#include "general.h"
#include "HeapSort.h"

/**************************************************************************/
/* Les defines et bricoles diverses
 */

#define CL1  1
#define CL2  2
#define CL12 3

/**************************************************************************/
/* Les fonctions globales pour le chunking
 */
void chunkerInit(void);

void setCstes(int the_chunk_size, real the_eps, int the_iter_min_to_shrink, int the_proportion_to_shrink, real the_cache_size_mega);
void setChunkSize(int the_chunk_size);
void setEps(real the_eps);
void setIterMinToShrink(int the_iter_min_to_shrink);
void setProportionToShrink(int the_proportion_to_shrink);
void setCacheSize(int the_cache_size_mega);
void setIPrecision(real the_i_precision);

void fatalError(char *str);

#ifdef I_WANT_TIME
long getRuntime(void);
#endif

void solve(real *the_alpha, int the_n, int the_width);
void Rsolve(real *the_alpha, int the_n, int the_width);

/**************************************************************************/
/* Les variables globales pour le chunking
 */

int n_CL1;
int n_CL2;

int *active_var;
int *class_var;
real *accel;
int *select_var;
int n_select_var;
int n_active_var;
int n;
real *alpha;
real *alpha_new;
int chunk_size;

/* Internal Optimizer :
	QCMoptimizer *i_opti;*/
bool i_opti_is_set;

/* Buffers :
 */
real *r_buffer;
int  *i_buffer;

/* Cache handling :
 */
int cache_size;
int n_total_cache;
int n_free_cache;
real *mem_cache;
real **ptr_free_cache;
real **row_ptr_cache;
int *n_use_cache;
int *query_cache;
bool *i_need_it;

/* Sub Problem
 */
real *K_SP;
/*real *K_SP_data;
 */
real *b_SP;
real *Y_SP;
real i_precision;
    
/* Internal Loop
 */
bool its_the_end;
real lb;
int iteration;

/* Skrink
 */
int *not_at_bound_at;
int iter_min_to_shrink;
int n_min_to_shrink;
int *bsv;
int *nulls;
int n_bsv;
int n_nulls;
int proportion_to_shrink;
bool we_have_yet_shrink;
		
/* Various :
 */
real eps, Ceps;

/* Special Regression
 */
int *active_mod_var;
int n_active_mod_var;
int *atomise_cache;
real *Y_SP;
real *CL_SP;
bool *is_registered;


/* Special One adapted from bloody fu****g C++
 */
real *points;
real *Y;
bool debug;
real precision;
real C;
int width;
real epsilon;

real (*kernel)(real *x, real *y);
void (*launchSubProblem)(void);

#endif
